#!/usr/bin/perl

# csv_pixel_triplets_to_G-code_coords.pl
# take a path encoded in a CSV file as a sequence of pixel locations and gray
#  levels and turn it into a path encoded as a sequence of "moves" in G-code

# Approach:
# write a preamble

# presumably move to retract height, move to location above first point
# lower to safe height above terrain
# while (not end-of-path), G1 move to next point
# raise G0 to retract height

# write an epilogue 

$mz_safe = 2.5;
$feedrate = 30;

sub find_pixel_mins_maxs {
  my $px_min=9999999;
  my $px_max=-999999;
  my $py_min=9999999;
  my $py_max=-999999;
  my $g_min=9999999; # graylevel, 0<=g<=255
  my $g_max=-999999;
  @files = glob("path*.csv");

  $first=1; # flag for first pair
  foreach $file (@files) {
    print "Looking for extreme points: $file\n" if -f $file;
    open FILEIN, "<$file" or die $!;

    while (<FILEIN>) {
      @triplets = split /\,/ , $_; # split on comma
      $px=$triplets[0]; if($px>$px_max) {$px_max=$px;}; if($px<$px_min) {$px_min=$px;};
      $py=$triplets[1]; if($py>$py_max) {$py_max=$py;}; if($py<$py_min) {$py_min=$py;};
      $g=$triplets[2]; if($g>$g_max) {$g_max=$g;}; if($g<$g_min) {$g_min=$g;};
      if($first) {$px_first=$px; $py_first=$py; $first=0; }
    }

    printf "Pixel extrema: %.3f<=X<=%.3f\n               %.3f<=Y<=%.3f\n               %.3f<=G<=%.3f\n",$px_min,$px_max,$py_min,$py_max,$g_min,$g_max;
  }
  close FILEIN;

  return ($px_first,$py_first,$px_min,$px_max, $py_min,$py_max, $g_min,$g_max);
} # endsub find_pixel_mins_maxs


sub write_preamble {
  local($mx_first,$my_first) = ($_[0],$_[1]);
  local($mx_inches,$my_inches,$mz_inches) = ($_[2],$_[3],$_[4]);
  local($mx_center,$my_center,$mz_center) = ($_[5],$_[6],$_[7]);
  print FILEOUT <<"EOT";
%
(MSG,G-code created by pixel_triplets_to_G-code_coords.pl script.)
(MSG,PATHNAME: ".")
(MSG,FILENAME: output.ngc)
(MSG, STOCK/BLOCK,$mx_inches,$my_inches,$mz_inches,$mx_center,$my_center,$mz_center) 
(STOCK/BLOCK,$mx_inches,$my_inches,$mz_inches,$mx_center,$my_center,$mz_center) 
(set Path-blending tolerance)
G64 P0.001
(set inch units)
G20
(set abs. distance mode)
G90
(TOOL/MILL,0.25000,0,1.5000,0.0)
M6 T10125
EOT
  printf FILEOUT "G00 Z%f\n",$mz_safe;
  printf FILEOUT "G00 X%.3f Y%.3f\n",$mx_first,$my_first;
  printf FILEOUT "F%f\n",$feedrate;
} # endsub write_preamble

sub write_postamble {
  local($a,$b) = ($_[0],$_[1]);
  printf FILEOUT "G00 Z%d\n",$mz_safe;
  print FILEOUT <<"EOT";
M5
M2
%
EOT
} # endsub write_postamble

# Query for x,y,z values in pixels and inches
print "Enter image width PX in pixels:";
$px_pixels =  <STDIN>;
chomp ($px_pixels);
print "Enter height PY in pixels:";
$py_pixels =  <STDIN>;
chomp ($py_pixels);
print "Enter brightest graylevel (0-255):";
$g_max =  <STDIN>;
chomp ($g_max);
print "Enter dimmest graylevel (0-255):";
$g_min=  <STDIN>;
chomp ($g_min);
$g_range = $g_max - $g_min;
print "Summarizing: Size in pixels and gray levels\n";
print "             px:$px_pixels\px py:$py_pixels\px\n";
print "             g_min:$g_min g_max:$g_max\n";
print "             g_range:$g_range\n";
print "Now enter model width MX in inches:";
$mx_inches =  <STDIN>;
chomp ($mx_inches);
$x_scale=$px_pixels/$mx_inches;
print "And MY in inches:";
$my_inches =  <STDIN>;
chomp ($my_inches);
$y_scale=$py_pixels/$my_inches;
print "And MZ in inches:";
$mz_inches =  <STDIN>;
chomp ($mz_inches);
$z_scale=$g_range/$mz_inches;
print "Summarizing: Model size in inches\n";
print "             mx:$mx_inches\in  my:$my_inches\in  mz:$mz_inches\in\n";
print "Calculated scale factors\n";
printf "             Xs:%3.3fpx/in  Ys:%3.3fpx/in  Zs:%3.3fglevels/in\n",$x_scale,$y_scale,$z_scale;

($px_first,$py_first, $px_min,$px_max, $py_min,$py_max, $g_min,$g_max)=&find_pixel_mins_maxs;
print "Consider these when answering queries.\n";

print "Press ENTER to continue:";
<STDIN>;

open FILEOUT, ">output.ngc" or die $!;

# zero at middle top of model
$mx_center=$mx_inches/2.0;
$my_center=$my_inches/2.0;
$mz_center=$mz_inches;
$mx_first=$px_first/$x_scale-$mx_center;
# NB Be ware that py=0 is top-of-image, my=0 is bottom
$my_first=($py_pixels-$py_first)/$y_scale-$my_center;
&write_preamble($mx_first,$my_first,$mx_inches,$my_inches,$mz_inches,$mx_center,$my_center,$mz_center,);

@files = glob("path*.csv");
foreach $file (@files) {
  print "Looking for extreme points: $file\n" if -f $file;
  open FILEIN, "<$file" or die $!;

  while (<FILEIN>) {
    @triplets = split /\,/ , $_; # split on comma
    $px=$triplets[0];
    $py=$triplets[1];
    $g=$triplets[2];
    $mx=$px/$x_scale-$mx_center;
    # NB Be ware that py=0 is top-of-image, my=0 is bottom
    $my=($py_pixels-$py)/$y_scale-$my_center;
    $mz=$g/$z_scale-$mz_center;

    printf FILEOUT "G01 X%.3f Y%.3f Z%.3f\n",$mx,$my,$mz;
  }

  close FILEIN;
}

&write_postamble;

print "Output file output.ngc written.\nExiting.\n";

close FILEOUT;

exit;
